@@ -1,3 +1,4 @@
+
 import pandas as pd
 import requests
 import json
@@ -35,6 +36,8 @@ def expand_df(df,key='',value=''):
     
   return df
 
+
+
 def get_fixtures(no_results=100,page=0,compSeasons=0,gameweeks=0,teams='',statuses='U,L,C',startDate='',endDate='',sort='asc'):
 
   url = "https://footballapi.pulselive.com/football/fixtures"
@@ -519,6 +522,7 @@ class SofaScore:
     res = expand_df(pd.DataFrame(res))
     return res
 
+
   # Get a list of players (and their basic stats)
   def get_players(tournament=17, season=29415,limit=20,order='-rating',offset=20,group='summary'):
 
@@ -545,10 +549,10 @@ class SofaScore:
       res = json.loads(response.text.encode('utf8'))
       return res
     except:
-      pass
+      return {"error":"oops"}
 
 
-  # Get detailed stats for one player
+  # Player stats for a  Tournament Season 
   def player_stats(tournament=17,player_id=43443,season=29415):
 
     url = f"https://api.sofascore.com/api/v1/player/{player_id}/unique-tournament/{tournament}/season/{season}/statistics/overall"
@@ -574,7 +578,7 @@ class SofaScore:
     res = json.loads(response.text.encode('utf8'))
     return res
 
-  # Get player's heatmap in x,y coordinates
+  # Get player's heatmap for the entire season in x,y coordinates
   def heatmap(player_id=12994,tournamen=7,season=29267):
     url = f"https://api.sofascore.com/api/v1/player/{player_id}/unique-tournament/{tournamen}/season/{season}/heatmap/overall"
 
@@ -597,4 +601,281 @@ class SofaScore:
     response = requests.request("GET", url, headers=headers, data = payload)
 
     res = json.loads(response.text.encode('utf8'))
-    return res
\ No newline at end of file
+    return res
+
+
+  # List of matches for a tournament season
+  def matches(tournament_id=7,season_id=23766,page=0):
+    url = f"https://api.sofascore.com/api/v1/unique-tournament/{tournament_id}/season/{season_id}/events/last/{page}"
+
+    payload={}
+    headers = {
+      'authority': 'api.sofascore.com',
+      'pragma': 'no-cache',
+      'cache-control': 'no-cache',
+      'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 11_0_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.198 Safari/537.36',
+      'accept': '*/*',
+      'origin': 'https://www.sofascore.com',
+      'sec-fetch-site': 'same-site',
+      'sec-fetch-mode': 'cors',
+      'sec-fetch-dest': 'empty',
+      'referer': 'https://www.sofascore.com/',
+      'accept-language': 'en-GB,en-US;q=0.9,en;q=0.8',
+      'Cookie': ''
+    }
+
+    response = requests.request("GET", url, headers=headers, data=payload)
+    try:
+      res = json.loads(response.text.encode('utf8'))
+      return res
+    except:
+      pass
+
+
+  # Match Details
+  def match_details(match_id=8197187):
+
+    url = f"https://api.sofascore.com/api/v1/event/{match_id}"
+
+    payload={}
+    headers = {
+      'authority': 'api.sofascore.com',
+      'pragma': 'no-cache',
+      'cache-control': 'no-cache',
+      'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 11_0_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.198 Safari/537.36',
+      'accept': '*/*',
+      'origin': 'https://www.sofascore.com',
+      'sec-fetch-site': 'same-site',
+      'sec-fetch-mode': 'cors',
+      'sec-fetch-dest': 'empty',
+      'referer': 'https://www.sofascore.com/',
+      'accept-language': 'en-GB,en-US;q=0.9,en;q=0.8',
+      'Cookie': ''
+    }
+
+    response = requests.request("GET", url, headers=headers, data=payload)
+
+    try:
+      res = json.loads(response.text.encode('utf8'))
+      return res
+    except:
+      pass
+
+
+  # Match Graph per minute
+  def match_graph(match_id=8197187):
+
+    url = f"https://api.sofascore.com/api/v1/event/{match_id}/graph"
+
+    payload={}
+    headers = {
+      'authority': 'api.sofascore.com',
+      'pragma': 'no-cache',
+      'cache-control': 'no-cache',
+      'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 11_0_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.198 Safari/537.36',
+      'accept': '*/*',
+      'origin': 'https://www.sofascore.com',
+      'sec-fetch-site': 'same-site',
+      'sec-fetch-mode': 'cors',
+      'sec-fetch-dest': 'empty',
+      'referer': 'https://www.sofascore.com/',
+      'accept-language': 'en-GB,en-US;q=0.9,en;q=0.8',
+      'Cookie': ''
+    }
+
+    response = requests.request("GET", url, headers=headers, data=payload)
+
+    try:
+      res = json.loads(response.text.encode('utf8'))
+      return res
+    except:
+      pass
+
+
+
+  # Match Stats
+  def match_stats(match_id=8197187):
+
+    url = f"https://api.sofascore.com/api/v1/event/{match_id}/statistics"
+
+    payload={}
+    headers = {
+      'authority': 'api.sofascore.com',
+      'pragma': 'no-cache',
+      'cache-control': 'no-cache',
+      'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 11_0_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.198 Safari/537.36',
+      'accept': '*/*',
+      'origin': 'https://www.sofascore.com',
+      'sec-fetch-site': 'same-site',
+      'sec-fetch-mode': 'cors',
+      'sec-fetch-dest': 'empty',
+      'referer': 'https://www.sofascore.com/',
+      'accept-language': 'en-GB,en-US;q=0.9,en;q=0.8',
+      'Cookie': ''
+    }
+
+    response = requests.request("GET", url, headers=headers, data=payload)
+
+    try:
+      res = json.loads(response.text.encode('utf8'))
+      return res
+    except:
+      pass
+
+
+  #Lineup
+  def match_lineup(match_id=8197187):
+    url = f"https://api.sofascore.com/api/v1/event/{match_id}/lineups"
+
+    payload={}
+    headers = {
+      'authority': 'api.sofascore.com',
+      'pragma': 'no-cache',
+      'cache-control': 'no-cache',
+      'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 11_0_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.198 Safari/537.36',
+      'accept': '*/*',
+      'origin': 'https://www.sofascore.com',
+      'sec-fetch-site': 'same-site',
+      'sec-fetch-mode': 'cors',
+      'sec-fetch-dest': 'empty',
+      'referer': 'https://www.sofascore.com/',
+      'accept-language': 'en-GB,en-US;q=0.9,en;q=0.8',
+      'Cookie': ''
+    }
+
+    response = requests.request("GET", url, headers=headers, data=payload)
+
+    try:
+      res = json.loads(response.text.encode('utf8'))
+      return res
+    except:
+      pass
+
+
+  # Incidents in a match (Eg. injury, goal)
+  def match_incidents(match_id=8197187):
+
+    url = f"https://api.sofascore.com/api/v1/event/{match_id}/incidents"
+
+    payload={}
+    headers = {
+      'authority': 'api.sofascore.com',
+      'pragma': 'no-cache',
+      'cache-control': 'no-cache',
+      'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 11_0_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.198 Safari/537.36',
+      'accept': '*/*',
+      'origin': 'https://www.sofascore.com',
+      'sec-fetch-site': 'same-site',
+      'sec-fetch-mode': 'cors',
+      'sec-fetch-dest': 'empty',
+      'referer': 'https://www.sofascore.com/',
+      'accept-language': 'en-GB,en-US;q=0.9,en;q=0.8',
+      'Cookie': ''
+    }
+
+    response = requests.request("GET", url, headers=headers, data=payload)
+
+    try:
+      res = json.loads(response.text.encode('utf8'))
+      return res
+    except:
+      pass
+
+  # Player's 5 basic attributes
+  def player_attribute_overview(player_id=12994):
+
+    url = f"https://api.sofascore.com/api/v1/player/{player_id}/attribute-overviews"
+
+    payload={}
+    headers = {
+      'authority': 'api.sofascore.com',
+      'pragma': 'no-cache',
+      'cache-control': 'no-cache',
+      'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 11_0_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.198 Safari/537.36',
+      'accept': '*/*',
+      'origin': 'https://www.sofascore.com',
+      'sec-fetch-site': 'same-site',
+      'sec-fetch-mode': 'cors',
+      'sec-fetch-dest': 'empty',
+      'referer': 'https://www.sofascore.com/',
+      'accept-language': 'en-GB,en-US;q=0.9,en;q=0.8',
+      'Cookie': ''
+    }
+
+    response = requests.request("GET", url, headers=headers, data=payload)
+
+    try:
+      res = json.loads(response.text.encode('utf8'))
+      return res
+    except:
+      pass
+
+  # All matches (/events) of a player
+  def player_events(player_id=12994,page=0):
+    url = f"https://api.sofascore.com/api/v1/player/{player_id}/events/last/{page}"
+
+    payload={}
+    headers = {
+      'authority': 'api.sofascore.com',
+      'pragma': 'no-cache',
+      'cache-control': 'no-cache',
+      'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 11_0_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.198 Safari/537.36',
+      'accept': '*/*',
+      'origin': 'https://www.sofascore.com',
+      'sec-fetch-site': 'same-site',
+      'sec-fetch-mode': 'cors',
+      'sec-fetch-dest': 'empty',
+      'referer': 'https://www.sofascore.com/',
+      'accept-language': 'en-GB,en-US;q=0.9,en;q=0.8',
+      'Cookie': ''
+    }
+
+    response = requests.request("GET", url, headers=headers, data=payload)
+
+    try:
+      res = json.loads(response.text.encode('utf8'))
+      return res
+    except:
+      pass
+
+  # Characterstics of a player
+  def player_characterstics(player_id=12994):
+
+    url = f"https://api.sofascore.com/api/v1/player/{player_id}/characteristics"
+
+    payload={}
+    headers = {
+      'authority': 'api.sofascore.com',
+      'pragma': 'no-cache',
+      'cache-control': 'no-cache',
+      'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 11_0_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.198 Safari/537.36',
+      'accept': '*/*',
+      'origin': 'https://www.sofascore.com',
+      'sec-fetch-site': 'same-site',
+      'sec-fetch-mode': 'cors',
+      'sec-fetch-dest': 'empty',
+      'referer': 'https://www.sofascore.com/',
+      'accept-language': 'en-GB,en-US;q=0.9,en;q=0.8',
+      'Cookie': ''
+    }
+
+    response = requests.request("GET", url, headers=headers, data=payload)
+
+    try:
+      res = json.loads(response.text.encode('utf8'))
+      return res
+    except:
+      pass
+
+
+
+
+
+
+
+
+
+
+
+
+
